# frozen_string_literal: true

require "avatax_client/errors"
require "avatax_client/reverse_coercion"

# Base class for handling requests from Avatax.
class AvataxClient::Request::Base < Hashie::Dash
  include Hashie::Extensions::Dash::PropertyTranslation
  include Hashie::Extensions::Dash::Coercion
  include AvataxClient::ReverseCoercion
  include Hashie::Extensions::IgnoreUndeclared

  ## ClassMethods
  class << self
    private

    # Provide a hook to allow user to debug a response.
    # @param response [Typhoeus::Response] response from api call.
    # @return [void]
    def check_debug(response:)
      if AvataxClient.configuration.debug
        AvataxClient.configuration.debug_response.call(response)
      end
    end

    # @param response [Typhoeus::Response] response from api call.
    # @return [AvataxClient::Response, AvataxClient::Errors]
    def handle_response(response:)
      check_debug(response: response)

      hash = JSON.parse(response.body)
      if response.success?
        to_s.gsub("Request", "Response").constantize.new(hash)
      elsif response.timed_out?
        raise AvataxClient::Errors::TimeoutError
      else
        msg = hash.dig("error", "code") || "Error"
        raise AvataxClient::Errors::ApiError.new(hash), msg
      end
    end

    def headers
      { "Content-Type" => "application/json", "Accept" => "application/json" }
    end

    def credentials
      avatax = AvataxClient.configuration
      "#{avatax.username}:#{avatax.password}"
    end

    def url(path)
      config = AvataxClient.configuration
      endpoint = config.endpoint
      version = config.version

      URI.join(endpoint, ["api", version, path].join("/")).to_s
    end
  end
end
