# frozen_string_literal: true

# Handles requests to Avatax api dealing with Transactions.
class AvataxClient::Request::Transaction < AvataxClient::Request::Base
  # only required if needing to set an Address for the entire transaction
  property :addresses, coerce: Array[AvataxClient::Request::Address], default: []
  property :code
  property :commit
  property :company_code, from: :companyCode, required: true
  property :customer_code, from: :customerCode, required: true
  property :date, required: true
  property :lines, required: true, coerce: Array[AvataxClient::Request::LineItem]
  property :type

  ## ClassMethods
  class << self
    # Reasons to make an adjustment to a Avatax Transaction.
    # @return [OpenStruct]
    #
    # rubocop:disable Metrics/MethodLength
    def adjustment_reasons
      OpenStruct.new(
        bad_debt: "BadDebt",
        exempt_cert: "ExemptCertApplied",
        not_adjusted: "NotAdjusted",
        offline: "Offline",
        other: "Other",
        price_adjusted: "PriceAdjusted",
        product_exchanged: "ProductExchanged",
        product_returned: "ProductReturned",
        reconciled: "ReconciledWithGeneralLedger",
        sourcing_issue: "SourcingIssue"
      )
    end
    # rubocop:enable Metrics/MethodLength

    # @param adjustment_reason [String] @see http://developer.avalara.com/avatax/api-reference/tax/v2/Transactions/#ApiV2CompaniesByCompanyCodeTransactionsByTransactionCodeAdjustPost
    # @param adjustment_description [String] description as to why the adjustment was made.
    # @param transaction [AvataxClient::Request::Transaction]
    # @return (see AvataxClient::Request::Base.handle_response)
    def adjust!(adjustment_reason:, adjustment_description: "", transaction:)
      body_params = {
        adjustmentReason: adjustment_reason,
        adjustmentDescription: adjustment_description,
        newTransaction: transaction.to_body
      }
      body = JSON.generate(body_params.delete_if { |_key, value| value.blank? })

      path = "companies/#{transaction.company_code}/transactions/#{transaction.code}/adjust"
      res = Typhoeus.post(url(path), body: body, headers: headers, userpwd: credentials)
      handle_response(response: res)
    end

    # @param transaction [AvataxClient::Request::Transaction]
    # @return (see AvataxClient::Request::Base.handle_response)
    def create!(transaction:)
      path = "transactions/create"
      hash = transaction.to_body
      body = JSON.generate(hash)
      res = Typhoeus.post(url(path), body: body, headers: headers, userpwd: credentials)
      handle_response(response: res)
    end

    # Changes an uncommitted transaction into a committed transaction.
    # @param company_code [String] unique code identifying the company
    # @param transaction_code [String] unique code identifying the transaction
    # @return (see AvataxClient::Request::Base.handle_response)
    def commit!(company_code:, transaction_code:)
      path = "companies/#{company_code}/transactions/#{transaction_code}/commit"
      body = JSON.generate(commit: true)
      res = Typhoeus.post(url(path), body: body, headers: headers, userpwd: credentials)
      handle_response(response: res)
    end

    # Commenting out for now. Need to add tests and circle back.

    # Deletes a transaction in Avalara by uncommitting it.
    # @param company_code [String] identifies the Company tax profile in Avalara
    # @param transaction_code [String] unique identifier of the transaction
    # @return [boolean]
    # def void(company_code:, transaction_code:)
    #   params = { code: "DocDeleted" }
    #   path = "companies/#{company_code}/transactions/#{transaction_code}/void"
    #   body = JSON.generate(params)
    #   Typhoeus.post(url(path), body: body, headers: headers, userpwd: credentials)
    # end

    # def adjust(adjustment_reason:, adjustment_description: "", company_code:, transaction:, transaction_code:)
    #   body_params = {
    #     adjustmentReason: adjustment_reason,
    #     adjustmentDescription: adjustment_description,
    #     newTransaction: transaction.build_body
    #   }
    #   body = JSON.generate(body_params.delete_if { |_key, value| value.blank? })

    #   path = "companies/#{company_code}/transactions/#{transaction_code}/adjust"
    #   res = Typhoeus.post(url(path), body: body, headers: headers, userpwd: credentials)
    #   handle_response(response: res)
    # end

    # def change_code(company_code:, transaction_code:, new_code:)
    #   path = "companies/#{company_code}/transactions/#{transaction_code}/changecode"
    #   body_params = { "newCode" => new_code }
    #   body = JSON.generate(body_params)
    #   res = Typhoeus.post(url(path), body: body, headers: headers, userpwd: credentials)
    #   handle_response(response: res)
    # end

    # def get(company_code:, transaction_code:, include: "")
    #   path = "companies/#{company_code}/transactions/#{transaction_code}"
    #   res = Typhoeus.get(url(path), headers: headers, params: { "$include" => include }, userpwd: credentials)
    #   handle_response(response: res)
    # end

    # def find_by_id(id:)
    #   path = "transactions/#{id}"
    #   res = Typhoeus.get(url(path), headers: headers, userpwd: credentials)
    #   handle_response(response: res)
    # end
  end

  ## Instance methods

  # @see {.adjust!}
  def adjust!(adjustment_description: "", adjustment_reason:)
    self.class.adjust!(adjustment_description: adjustment_description, adjustment_reason: adjustment_reason,
                       transaction: self)
  end

  # @see {.create!}
  def create!
    self.class.create!(transaction: self)
  end

  # @see {.commit!}
  def commit!
    self.class.commit!(company_code: company_code, transaction_code: code)
  end
end
